@@ -99,6 +99,7 @@ sub _dec_semurl {
 
 sub DisplayOfURL {
     my $url = shift;
+    my $dev_mode = shift;
 
     return $url unless
         $url =~ m!^https?://([^/]+)(/.*)?$!;
@@ -106,6 +107,11 @@ sub DisplayOfURL {
     my ($host, $path) = ($1, $2);
     $host = lc($host);
 
+    if ($dev_mode) {
+        $host =~ s!^dev\.!!;
+        $host =~ s!:\d+!!;
+    }
+
     $host =~ s/:.+//;
     $host =~ s/^www\.//i;
 
@@ -119,6 +125,13 @@ sub DisplayOfURL {
         return "$1 [$host]";
     }
 
+    if ($host =~ m!^profile\.(.+)!i) {
+        my $site = $1;
+        if ($path =~ m!^/([^/]+)/?$!) {
+            return "$1 [$site]";
+        }
+    }
+
     return $url;
 }
 
